@@ -33,6 +33,7 @@
 #include "camel-exchange-search.h"
 #include "camel-exchange-store.h"
 #include "camel-exchange-summary.h"
+#include "camel-exchange-journal.h"
 
 #include <camel/camel-data-wrapper.h>
 #include <camel/camel-exception.h>
@@ -44,7 +45,7 @@
 #include <camel/camel-stream-filter.h>
 #include <camel/camel-stream-mem.h>
 
-static CamelFolderClass *parent_class = NULL;
+static CamelOfflineFolderClass *parent_class = NULL;
 
 /* Returns the class for a CamelFolder */
 #define CF_CLASS(so) CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(so))
@@ -52,8 +53,7 @@ static CamelFolderClass *parent_class = NULL;
 static void refresh_info (CamelFolder *folder, CamelException *ex);
 static void folder_sync (CamelFolder *folder, gboolean expunge,
 			 CamelException *ex);
-static void expunge             (CamelFolder *folder,
-				 CamelException *ex);
+static void exchange_expunge (CamelFolder *folder, CamelException *ex);
 static void append_message (CamelFolder *folder, CamelMimeMessage *message,
 			    const CamelMessageInfo *info, char **appended_uid,
 			    CamelException *ex);
@@ -81,22 +81,26 @@ static void   transfer_messages_the_hard_way (CamelFolder *source,
 					      GPtrArray **transferred_uids,
 					      gboolean delete_originals,
 					      CamelException *ex);
+static void exchange_refresh_info (CamelFolder *folder, CamelException *ex);
+static void exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex);
 
 static void
 class_init (CamelFolderClass *camel_folder_class)
 {
-	parent_class = (CamelFolderClass *)camel_type_get_global_classfuncs (camel_folder_get_type ());
+	parent_class = CAMEL_OFFLINE_FOLDER_CLASS (camel_type_get_global_classfuncs (camel_offline_folder_get_type ())) ;
 
 	/* virtual method definition */
 	camel_folder_class->sync = folder_sync;
 	camel_folder_class->refresh_info = refresh_info;
-	camel_folder_class->expunge = expunge;
+	camel_folder_class->expunge = exchange_expunge;
 	camel_folder_class->append_message = append_message;
 	camel_folder_class->get_message = get_message;
 	camel_folder_class->search_by_expression = search_by_expression;
 	camel_folder_class->search_by_uids = search_by_uids;
 	camel_folder_class->search_free = search_free;
 	camel_folder_class->transfer_messages_to = transfer_messages_to;
+	camel_folder_class->refresh_info = exchange_refresh_info;
+	camel_folder_class->sync = exchange_sync;
 }
 
 #define CAMEL_EXCHANGE_SERVER_FLAGS \
@@ -139,7 +143,8 @@ camel_exchange_folder_get_type (void)
 
 	if (camel_exchange_folder_type == CAMEL_INVALID_TYPE) {
 		camel_exchange_folder_type = camel_type_register (
-			CAMEL_FOLDER_TYPE, "CamelExchangeFolder",
+			camel_offline_folder_get_type (),
+			"CamelExchangeFolder",
 			sizeof (CamelExchangeFolder),
 			sizeof (CamelExchangeFolderClass),
 			(CamelObjectClassInitFunc) class_init,
@@ -201,7 +206,7 @@ refresh_info (CamelFolder *folder, CamelException *ex)
 }
 
 static void
-expunge (CamelFolder *folder, CamelException *ex)
+exchange_expunge (CamelFolder *folder, CamelException *ex)
 {
 	CamelExchangeFolder *exch = CAMEL_EXCHANGE_FOLDER (folder);
 	CamelFolder *trash;
@@ -268,7 +273,12 @@ append_message (CamelFolder *folder, CamelMimeMessage *message,
 		CamelException *ex)
 {
 	CamelStream *stream_mem;
+	CamelOfflineStore *offline = (CamelOfflineStore *) folder->parent_store;
 
+	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		camel_exchange_journal_append ((CamelExchangeJournal *) ((CamelExchangeFolder *)folder)->journal, message, info, appended_uid, ex);
+		return;
+	}
 	stream_mem = camel_stream_mem_new ();
 	camel_data_wrapper_write_to_stream (CAMEL_DATA_WRAPPER (message),
 					    stream_mem);
@@ -580,12 +590,41 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 {
 	CamelExchangeFolder *exch_source = CAMEL_EXCHANGE_FOLDER (source);
 	CamelExchangeFolder *exch_dest = CAMEL_EXCHANGE_FOLDER (dest);
+	CamelOfflineStore *offline = (CamelOfflineStore *) source->parent_store;
+	CamelMessageInfo *info;
 	GPtrArray *ret_uids = NULL;
-	int hier_len;
+	int hier_len, i;
 
 	camel_operation_start (NULL, delete_originals ? _("Moving messages") :
 			       _("Copying messages"));
 
+	/* Check for offline operation */
+	if (offline->state == CAMEL_OFFLINE_STORE_NETWORK_UNAVAIL) {
+		CamelExchangeJournal *journal = (CamelExchangeJournal *) exch_dest->journal;
+		CamelMimeMessage *message;
+		
+		for (i = 0; i < uids->len; i++) {
+			info = camel_folder_summary_uid (source->summary, uids->pdata[i]);
+			if (!info)
+				continue;
+			
+			if (!(message = get_message (source, camel_message_info_uid (info), ex)))
+				break;
+
+			camel_exchange_journal_append (journal, message, info, NULL, ex);
+			camel_object_unref (message);
+
+			if (camel_exception_is_set (ex))
+				break;
+			
+			if (delete_originals)
+				camel_folder_set_message_flags (source, camel_message_info_uid (info),
+					CAMEL_MESSAGE_DELETED, CAMEL_MESSAGE_DELETED);
+		}
+		goto end;
+ 	}
+
+
 	hier_len = strcspn (source->full_name, "/");
 	if (strncmp (source->full_name, dest->full_name, hier_len) != 0) {
 		transfer_messages_the_hard_way (source, uids, dest,
@@ -617,7 +656,7 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 		}
 	} else if (transferred_uids)
 		*transferred_uids = NULL;
-
+end:
 	camel_operation_end (NULL);
 }
 
@@ -872,7 +911,7 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 {
 	CamelExchangeFolder *exch = (CamelExchangeFolder *)folder;
 	const char *short_name;
-	char *summary_file;
+	char *summary_file, *journal_file;
 	GPtrArray *summary, *uids;
 	GByteArray *flags;
 	guint32 folder_flags;
@@ -910,6 +949,16 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 		return FALSE;
 	}
 
+	journal_file = g_strdup_printf ("%s/journal", folder_dir);
+	exch->journal = camel_exchange_journal_new (exch, journal_file);
+	g_free (journal_file);
+	if (!exch->journal) {
+		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
+					_("Could not create journal for %s"),
+					name);
+		return FALSE;
+	}
+
 	exch->thread_index_to_message_id =
 		g_hash_table_new (g_str_hash, g_str_equal);
 
@@ -984,3 +1033,18 @@ camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 
 	return TRUE;
 }
+
+static void 
+exchange_refresh_info (CamelFolder *folder, CamelException *ex)
+{
+	refresh_info (folder, ex);
+}
+
+static void 
+exchange_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
+{
+	if (expunge)
+		exchange_expunge (folder, ex);
+	
+	camel_folder_summary_save (folder->summary);
+}
